import type { StaticImg } from '@/utils/static'
import { onLoad } from '@dcloudio/uni-app'
import { toRefs } from 'vue'
import { reactive } from 'vue'
import { inject, onUnmounted, onMounted, type ComponentInternalInstance, ref} from 'vue'
import { encode, decode} from 'js-base64'

/**
 * 它从 `inject` 函数返回 `StaticImg` 服务
 * 图片的key就是图片名称
 * @returns { StaticImg } assets下全部的文件
 * @example
 * const staticImg = useStaticImg()
 * const logo = staticImg.logo
 */
export const useStaticImg = (): StaticImg => inject<StaticImg>("staticImg")!

/**
 *
 * @description 重复执行的定时器， 会在组件的unMounted阶段销毁
 *
 */
export const useScheduler = (cb: () => void, delay: number) => {

  const stx: API.UseSchedulerFastCall = {
    timeId: null,
    start: () => {},
    pause: () => {},
    stop: () => {},
    onPlaying: () => {}
  }

  let timeId: number | null = null

  const playing = () => {}

  const clearScheduler = () => clearInterval(timeId as number)

  onUnmounted(clearScheduler)

  stx.start = () => {
    timeId = setInterval(() => {
      cb()
      playing()
    }, delay)
  }

  stx.pause = () => clearScheduler()

  stx.onPlaying = playing

  stx.stop = () => clearScheduler()

  return stx
}


/**
 * @description 执行一定的定时器， 会在执行完后销毁
 * @param cb  回调函数
 * @param delay 延迟 大部分浏览器setTimeout延迟默认在四毫秒左右
 */
export const useSchedulerOnce = (
  cb: () => void,
  delay: number = 4
): void => {
  const timeId = setTimeout(() => {
    cb()
    clearTimeout(timeId)
  }, delay)

  onUnmounted(() => clearTimeout(timeId))

}

/**
 * 它返回一个对象，其中包含导航栏的高度、导航栏内内容的高度以及导航栏的顶部位置。
 * @returns 具有 height、contentHeight 和 top 属性的对象。
 */
export const useNavbarInfo = () => {

  const { uniPlatform, safeArea , ...systemInfo} = uni.getSystemInfoSync()

  console.log(uniPlatform,  window['navigatorBarHeight'], window['statusBarHeight']);
  
  if (uniPlatform == "web") {
    const statusBarHeight = window['statusBarHeight'] ? window['statusBarHeight'] * 2 : 33
    return {
      height: 140 + statusBarHeight  + "rpx",
      contentHeight: '64rpx',
      top: 38 + statusBarHeight + 'rpx',
      left: '40rpx',
      safeAreaTop: statusBarHeight + 'rpx',
      statusBarHeight: statusBarHeight
    }
  
  } else {

    const bounding = uni.getMenuButtonBoundingClientRect()
    console.log("(safeArea!.top + bounding.height):", (safeArea!.top + bounding.height));
    
    return {
      height: 104 + 'px',
      contentHeight: bounding.height + (bounding.top - systemInfo.statusBarHeight!) * 2 + "px",
      top: systemInfo.statusBarHeight!  + "px",
      left: '20px',
      safeAreaTop: 0 + 'rpx',
      statusBarHeight: 0
    }
  }

}

/**
 * 它需要一个 id、一个回调函数和一个 vm，然后它使用具有给定 id 的元素的 boundingClientRect 调用回调函数
 * @param {string} id - 您要从中获取信息的元素的 ID
 * @param cb - 回调函数
 * @param {ComponentInternalInstance} vm - 组件实例
 */
export const useQueryElInfo = (
  id: string,
  cb: (params: UniApp.NodeInfo | UniApp.NodeInfo[] ) => void,
  vm?: ComponentInternalInstance,
  immediate: boolean = true
) => {

  const queryNode = () => {
    const query = vm ? uni.createSelectorQuery().in(vm) :  uni.createSelectorQuery();
    query.select(id).boundingClientRect(data => cb(data)).exec();
  }

  immediate ? onMounted(() => queryNode()) : queryNode()
}

/**
 * 它创建一个音频实例，并返回一个具有 destroy、play 和 onPlayend 方法的对象。
 * @param {string} src - 音频文件的来源
 * @returns 具有三个属性的对象：destroy、play 和 onPlayend。
 */
export const useAudioMange = (src?: string) => {

  const audioInstance = ref< UniApp.InnerAudioContext>()
 
  audioInstance.value = uni.createInnerAudioContext()

  if (src) {
    audioInstance.value.src = src
  }

  const destroy = () => audioInstance.value?.stop()

  const repeat = () => {
    
  }

  const play = (src: string) => {
    audioInstance.value!.src = src
    audioInstance.value?.play()
  }

  const onplayend = (cb: Function) => audioInstance.value?.onEnded(() => cb && cb())

  onUnmounted(() => {
    destroy()
  })

  const atx = {
    destroy,
    play,
    onplayend,
  }

  return atx

}

/**
 * 它以字符串形式返回设备平台。
 * @returns uniPlatform 作为 DEVICE.Platform 返回。
 */
export const usePlatform = (): DEVICE.Platform => {

  const { uniPlatform } = uni.getSystemInfoSync()

  return uniPlatform as DEVICE.Platform
  
}


/**
 * 获取多设备动态适配的比率
 * @example
 *  const rate = useAdaptationIpadAndPhone()
 *  node.width = node.width * rate * 2 + 'rpx
 */
export const useAdaptationIpadAndPhone = () => {
  
  const { windowWidth, windowHeight } = uni.getSystemInfoSync()

  const rate = windowWidth / windowHeight
  
  if ( rate > 0.6 ) { // 大于0.6 说明是平板
    return 667 / ( windowHeight )
  } else {
    return 1
  }

}

export const useQueryParmas = async () => {
  console.log(1);
  let state = reactive({})
  
  state = await new Promise( (resolve) => {
    onLoad(query => {
      console.log(3);
      resolve(query!)
    })
  })
  
  return {...toRefs(state)}
}


/**
 * 这是一个导出对象的TypeScript函数，有两个方法decode和encode，分别将Base64编码的字符串解码为JSON对象，将JSON对象编码为Base64编码的字符串。
 * @returns 正在返回名为“useBase64”的自定义挂钩。这个钩子有两个函数：`decode` 和 `encode`。 `decode` 函数将字符串作为输入并返回解析后的 JSON 对象。
 * `encode` 函数将 `API.P` 类型的对象作为输入并返回 base64 编码的字符串。
 */
export const useBase64 = () => {

  return {
    decode: <T>(URI: string): T => JSON.parse(decode(URI)),
    encode: (records: API.P): string => encode(JSON.stringify(records))
  }

}